function [perf, perf_10perc, perf_90perc, all_explanations, allinitialscores] = EDC_libsvm(bb_name, bb_eval, bb_eval_args, tr_data, tr_class, te_data, te_class, te_class_original, feature_names, output_f_name_begin, dir_output)


%% EDC: Explaining Documents' Classifications 
% Using libSVM or libLINEAR implementation:
%   See libSVM:     http://www.csie.ntu.edu.tw/~cjlin/libsvm/index.html
%       libLINEAR:  http://www.csie.ntu.edu.tw/~cjlin/liblinear/
%
% Input:
%   bb_name: either 'libSVM' or 'libLINEAR'
%   bb_eval: function name to make a prediction, 'svmpredict' or 'predict'
%   bb_eval_args: trained model
%   tr_data: preprocessed training data
%   tr_class: classes of the training data
%   te_data: preprocessed test data, these will be explained
%   te_class: classes of the test data (1 vs r)
%   te_class_original: original classes of the test data (instead of 1 or 0, for output)
%   feature_names: all words from the dictionary
%   output_f_name_begin: string to name first elements of output, eg name of the classification problem
%   dir_output: output folder
%
% Output:
%   perf: defined explanation evaluation metrics of test instances
%       PCC_te: test accuracy
%       PE: percentage of documents explained
%       avg_dur_expl: average duration to find one explanation
%       avg_dur_all: average duration to find all explanations
%       avg_first_explanation_size: average size of the first explanation
%       avg_nb_explanations: average number of explanations given
%   perf_10perc: 10th percentile instead of average metrics
%   perf_90perc: 90th percentile instead of average metrics
%   all_explanations: all the explanations
%   allinitialscores: all the initial output scores of the model


if (strcmp(bb_name,'libSVM'))
    [Yte_sim Acc_te prob_est] = feval(bb_eval,te_class,sparse(te_data),bb_eval_args);
else
    [Yte_sim Acc_te prob_est] = feval(bb_eval,te_class,sparse(te_data),bb_eval_args,'-b 1');
end
prob_est = prob_est * bb_eval_args.Label(1);
allinitialscores = prob_est * bb_eval_args.Label(1);
PCC_te = Acc_te(1);

%% Explain test data and gather performance assessments
% load test_EDC_trainedSVMsave 

to_explain_i = [1:size(te_data,1)]; % Explain all test instances
nb_instances_to_explain = size(to_explain_i,2);
t1_durations = NaN * ones(nb_instances_to_explain,1);
t2_durations = NaN * ones(nb_instances_to_explain,1);
smallest_explanation_sizes = NaN * ones(nb_instances_to_explain,1);
all_explanations = {};
nb_explanations = NaN * ones(nb_instances_to_explain,1);
nb_explanations_smallest =  NaN * ones(nb_instances_to_explain,1);
iterations_to_first_explanation = NaN * ones(nb_instances_to_explain,1);
original_classes_to_explain = te_class_original(to_explain_i);
for i=1:nb_instances_to_explain 
    % explain document i
    expl_i = to_explain_i(i);
    doc = te_data(expl_i,:);
    output_file = strcat(dir_output, output_f_name_begin,'_',bb_name,'_explanations.txt');
    [t1,t2,iteration,explanations] = EDC_libSVM_explain_doc(doc, expl_i, original_classes_to_explain(i), feature_names, output_file, bb_eval, bb_eval_args);
    iterations_to_first_explanation(i) = iteration;
    t1_durations(i) = t1;
    t2_durations(i) = t2;
    all_explanations{i} = explanations;
    nbExpl = size(explanations,2);
    nb_explanations(i) = nbExpl;
    smallest_size = 0;
    sizeExpls = zeros(nbExpl,1);
    for j=1:nbExpl
        expl_i_size = size(find(explanations{j}~=0),2);
        sizeExpls(j) = expl_i_size;
        if ((expl_i_size < smallest_size) || (smallest_size == 0))
            smallest_size = expl_i_size;
        end
    end
    nb_explanations_smallest(i) = length(find(sizeExpls<= min(sizeExpls)));
    smallest_explanation_sizes(i) = smallest_size;
end


%% Evaluate explanations test instances
% Performance for all test instances 
% (one might want to limit this to those of class label 1)
nb_te = size(Yte_sim,1);

inans = find(isnan(t1_durations));
i_not_inans = find(~isnan(t1_durations));
i_not_inans_t2 = find(~isnan(t2_durations));

% for given m, n, nirws, ntotal, ndocs the performance is:
avg_first_explanation_size = mean(smallest_explanation_sizes(find(smallest_explanation_sizes > 0)));
avg_nb_explanations = mean(nb_explanations(find(~isnan(nb_explanations))));
if (size(i_not_inans,1) > 0)
    PE = size(i_not_inans,1)/nb_te;
    avg_dur_expl = mean(t1_durations(i_not_inans));
    avg_nb_iterations_first = mean(iterations_to_first_explanation(i_not_inans));
    perc_90_nb_iterations_first = prctile(iterations_to_first_explanation(i_not_inans),90);
else
    PE = 0;
    avg_dur_expl = NaN;
    avg_nb_iterations_first = NaN;
    perc_90_nb_iterations_first = NaN;
end
if (size(i_not_inans_t2,1) > 0)
    avg_dur_all = mean(t2_durations(i_not_inans_t2));
else
    avg_dur_all = NaN;
end

perf.PCC = PCC_te;
perf.PE = PE;
perf.ADF = avg_dur_expl;
perf.ADA = avg_dur_all;
perf.AWS = avg_first_explanation_size;
perf.ANT = avg_nb_explanations;
perf.ANS = mean(nb_explanations_smallest);
perf.AvgItF = avg_nb_iterations_first;
perf.Perc90ItF = perc_90_nb_iterations_first;

% The 10th and 90th percentiles of these performance measures
% ADF
perf_10perc.ADF = prctile(t1_durations(i_not_inans),10);
perf_90perc.ADF = prctile(t1_durations(i_not_inans),90);
% ADA
perf_10perc.ADA = prctile(t2_durations(i_not_inans),10);
perf_90perc.ADA = prctile(t2_durations(i_not_inans),90);
% ANT
perf_10perc.ANT = prctile(nb_explanations(find(~isnan(nb_explanations))),10);
perf_90perc.ANT = prctile(nb_explanations(find(~isnan(nb_explanations))),90);
% ANS
perf_10perc.ANS = prctile(nb_explanations_smallest(find(~isnan(nb_explanations_smallest))),10);
perf_90perc.ANS = prctile(nb_explanations_smallest(find(~isnan(nb_explanations_smallest))),90);
% AWS
perf_10perc.AWS = prctile(smallest_explanation_sizes(find(smallest_explanation_sizes > 0)),10);
perf_90perc.AWS = prctile(smallest_explanation_sizes(find(smallest_explanation_sizes > 0)),90);



%% Write performances to file

filename = strcat(dir_output, output_f_name_begin,'_',bb_name,'_perfs.txt');
fidout = fopen(filename ,'a');
fprintf(fidout,'\n PCC_te \t PE \t ADF \t ADA \t AWS \t ANT \t ANS \tAvg Nb Its F\t90Perc Nb Its F\n');
fprintf(fidout,'Perf for all test instances: \n%g\t%g\t%g\t%g\t%g\t%g\t%g\t%g\t%g\n',...
    perf.PCC,perf.PE,perf.ADF,perf.ADA,perf.AWS,perf.ANT,perf.ANS,perf.AvgItF,perf.Perc90ItF);
fclose(fidout);

%% Display resutls in output
perf

fprintf(1,'Performances written to:\n%s\n\n',filename);
filename_expl = strcat(dir_output, output_f_name_begin,'_',bb_name,'_explanations.txt');
fprintf(1,'Explanations written to:\n%s\n\n',filename_expl);
